﻿using DemoApplication.ModelView;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Data;
using System.Windows.Input;

namespace ProjectAnalogParus
{
    public class ModelClass : INotifyPropertyChanged
    {
        #region Public Constructors

        public ModelClass()
        {
            FillData();
        }

        #endregion Public Constructors

        #region Command

        /// <summary>
        ///     Refresh all
        /// </summary>
        public ICommand RefreshCommand => new DelegateCommand(RefreshData);

        #endregion Command

        #region Private Fields

        private ICollectionView collView;

        private string search;

        #endregion Private Fields

        #region Public Properties

        public ObservableCollection<Student> Employes { get; set; }
        public ObservableCollection<Student> FilteredList { get; set; }

        /// <summary>
        /// Global filter
        /// </summary>
        public string Search
        {
            get => search;
            set
            {
                search = value;

                collView.Filter = e =>
                {
                    var item = (Student)e;
                    return item != null && ((item.LastName?.StartsWith(search, StringComparison.OrdinalIgnoreCase) ?? false)
                                            || (item.FirstName?.StartsWith(search, StringComparison.OrdinalIgnoreCase) ?? false));
                };

                collView.Refresh();

                FilteredList = new ObservableCollection<Student>(collView.OfType<Student>().ToList());

                OnPropertyChanged("Search");
                OnPropertyChanged("FilteredList");
            }
        }

        #endregion Public Properties

        #region Public Events

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion Public Events

        #region Private Methods

        /// <summary>
        /// Fill data
        /// </summary>
        private void FillData()
        {

            search = "";

            var employe = new List<Student>();      

            
            

            Employes = new ObservableCollection<Student>(employe.AsParallel().OrderBy(o => o.LastName));

            FilteredList = new ObservableCollection<Student>(Employes);
            collView = CollectionViewSource.GetDefaultView(FilteredList);

            OnPropertyChanged("Search");
            OnPropertyChanged("Employes");
            OnPropertyChanged("FilteredList");
        }

        private void OnPropertyChanged(string propertyname)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyname));
        }

        /// <summary>
        /// refresh data
        /// </summary>
        /// <param name="obj"></param>
        private void RefreshData(object obj)
        {
            FillData();
        }

        #endregion Private Methods
    }
}
